@@ -192,8 +192,8 @@ namespace dbarts {
     }
     delete [] scratch.cutPoints; scratch.cutPoints = NULL;
     
-    if (state.trees != NULL) for (size_t i = 0; i < control.numTrees; ++i) state.trees[i].~Tree();
-    ::operator delete(state.trees); state.trees = NULL;
+    if (state.trees != NULL) for (size_t i = control.numTrees; i > 0; ) state.trees[--i].~Tree();
+    ::operator delete (state.trees); state.trees = NULL;
     delete [] state.treeIndices; state.treeIndices = NULL;
     
     delete [] state.treeFits; state.treeFits = NULL;
@@ -483,9 +483,14 @@ namespace {
     const double** cutPoints = const_cast<const double**>(scratch.cutPoints);
     for (size_t i = 0; i < data.numPredictors; ++i) cutPoints[i] = NULL;
     
-    state.trees = static_cast<Tree*>(::operator new[] (control.numTrees * sizeof(Tree)));
+    state.trees = static_cast<Tree*>(::operator new (control.numTrees * sizeof(Tree)));
     state.treeIndices = new size_t[data.numObservations * control.numTrees];
     
+    
+    for (size_t i = 0; i < control.numTrees; ++i) {
+      new (state.trees + i) Tree(state.treeIndices + i * data.numObservations, data.numObservations, data.numPredictors);
+    }
+    
     if (control.numThreads > 1 && ext_mt_create(&fit.threadManager, control.numThreads) != 0) {
       ext_printMessage("Unable to multi-thread, defaulting to single.");
     }
@@ -561,7 +566,7 @@ namespace {
       if (numCuts < numCutsPerVariable) ext_throwError("Number of induced cut points in new predictor less than previous: old splits would be invalid.");
       if (numCuts > numCutsPerVariable) ext_issueWarning("Number of induced cut points in new predictor greater than previous: ignoring extra quantiles.");
     } else {
-      numCutsPerVariable = numCuts;
+      numCutsPerVariable = (uint32_t) numCuts;
       cutPoints = new double[numCuts];
     }
     
@@ -603,10 +608,6 @@ namespace {
     Data& data(fit.data);
     State& state(fit.state);
     
-    for (size_t i = 0; i < control.numTrees; ++i) {
-      new (state.trees + i) Tree(state.treeIndices + i * data.numObservations, data.numObservations, data.numPredictors);
-    }
-    
     size_t length = data.numObservations * control.numTrees;
     state.treeFits = new double[length];
     for (size_t offset = 0; offset < length; ++offset) state.treeFits[offset] = 0.0;
